#pragma once

#include "HEJ/PDF.hh"
#include "HEJ/MatrixElement.hh"
#include "HEJ/RNG.hh"

#include "JetParameters.hh"
#include "Process.hh"
#include "Beam.hh"
#include "Status.hh"
#include "ParticleProperties.hh"

namespace HEJ{
  class Event;
  class HiggsCouplingSettings;
  class ScaleGenerator;
}

//! Namespace for HEJ Fixed Order Generator
namespace HEJFOG{
  class EventGenerator{
  public:
    EventGenerator(
        Process process,
        Beam beam,
        HEJ::ScaleGenerator scale_gen,
        JetParameters jets,
        int pdf_id,
        double subl_change,
        unsigned int subl_channels,
        ParticlesPropMap particles_properties,
        HEJ::HiggsCouplingSettings Higgs_coupling,
        HEJ::RNG & ran
    );

    HEJ::Event gen_event();

    Status status() const {
      return status_;
    }

  private:
    HEJ::PDF pdf_;
    HEJ::MatrixElement ME_;
    HEJ::ScaleGenerator scale_gen_;
    Process process_;
    JetParameters jets_;
    Beam beam_;
    Status status_;
    double subl_change_;
    unsigned int subl_channels_;
    ParticlesPropMap particles_properties_;
    std::reference_wrapper<HEJ::RNG> ran_;
  };

}
