/** \file
 *  \brief Container for event weights
 *
 *  \authors   Jeppe Andersen, Tuomas Hapola, Marian Heil, Andreas Maier, Jennifer Smillie
 *  \date      2019
 *  \copyright GPLv2 or later
 */
#pragma once

#include <vector>

namespace HEJ{
  //! Collection of weights assigned to a single event
  /**
   * A number of member functions of the MatrixElement class return Weights
   * objects containing the squares of the matrix elements for the various
   * scale choices.
   */
  struct Weights {
    double central;
    std::vector<double> variations;

    Weights& operator*=(Weights const & other);
    Weights& operator*=(double factor);
    Weights& operator/=(Weights const & other);
    Weights& operator/=(double factor);
  };

  inline
  Weights operator*(Weights a, Weights const & b) {
    return a*=b;
  }
  inline
  Weights operator*(Weights a, double b) {
    return a*=b;
  }
  inline
  Weights operator*(double b, Weights a) {
    return a*=b;
  }
  inline
  Weights operator/(Weights a, Weights const & b) {
    return a/=b;
  }
  inline
  Weights operator/(Weights a, double b) {
    return a/=b;
  }
}
