/** \file
 *  \brief Define different types of events.
 *
 *  \authors   Jeppe Andersen, Tuomas Hapola, Marian Heil, Andreas Maier, Jennifer Smillie
 *  \date      2019
 *  \copyright GPLv2 or later
 */

#pragma once

#include "HEJ/utility.hh"

namespace HEJ{

  //! Namespace for event types
  namespace event_type{
    //! Possible event types
    enum EventType: size_t{
      FKL,                        /**< FKL-type event */
      unordered_backward,         /**< event with unordered backward emission */
      unordered_forward,          /**< event with unordered forward emission */
      nonHEJ,                     /**< event configuration not covered by HEJ */
      no_2_jets,                  /**< event with less than two jets */
      bad_final_state,            /**< event with an unsupported final state */
      unob = unordered_backward,
      unof = unordered_forward,
      first_type = FKL,
      last_type = bad_final_state
    };

    //! Event type names
    /**
     * For example, names[FKL] is the string "FKL"
     */
    static constexpr auto names = make_array(
      "FKL",
      "unordered backward",
      "unordered forward",
      "nonHEJ",
      "no 2 jets",
      "bad final state"
    );

    inline
    bool is_HEJ(EventType type) {
      switch(type) {
      case FKL:
      case unordered_backward:
      case unordered_forward:
        return true;
      default:
        return false;
      }
    }

    inline
    bool is_uno(EventType type) {
      return type == unordered_backward || type == unordered_forward;
    }

  }

}
