#include <random>
#include <algorithm>

#include "LHEF/LHEF.h"
#include "HEJ/stream.hh"
#include "HEJ/event_types.hh"
#include "HEJ/Event.hh"

namespace{
  constexpr double min_jet_pt = 30.;
  const fastjet::JetDefinition jet_def{fastjet::kt_algorithm, 0.4};
  using namespace HEJ::event_type;
  static const std::vector<EventType> results{
    unob,FKL,FKL,FKL,FKL,FKL,FKL,unob,FKL,unob,FKL,FKL,FKL,unof,FKL,unob,FKL,
    FKL,unob,unob,FKL,FKL,unob,FKL,FKL,FKL,FKL,FKL,FKL,FKL,FKL,FKL,FKL,FKL,unof,
    FKL,FKL,unof,FKL,FKL,FKL,FKL,FKL,unof,FKL,FKL,FKL,unof,FKL,FKL,unob,unof,
    FKL,unof,FKL,unob,FKL,FKL,unob,FKL,unob,unof,unob,unof,FKL,FKL,FKL,FKL,FKL,
    FKL,FKL,FKL,FKL,FKL,FKL,FKL,unob,FKL,FKL,FKL,FKL,FKL,FKL,FKL,FKL,unob,FKL,
    FKL,FKL,FKL,unof,FKL,unob,FKL,FKL,FKL,FKL,FKL,FKL,FKL,FKL,FKL,FKL,unob,FKL,
    FKL,FKL,FKL,FKL,unob,FKL,unob,unob,FKL,FKL,FKL,FKL,FKL,FKL,FKL,FKL,FKL,unof,unob,FKL
  };
}

int main(int argn, char** argv) {
  if(argn != 2){
    std::cerr << "Usage: test_classify eventfile";
    return EXIT_FAILURE;
  }

  HEJ::istream in{argv[1]};
  LHEF::Reader reader{in};
  LHEF::Writer writer{std::cerr};
  writer.heprup = reader.heprup;
  std::mt19937 ran{};

  for(auto const & expected: results){
    reader.readEvent();
    HEJ::UnclusteredEvent uev{reader.hepeup};
    /*
     * Check if classification is independent of the ordering in the input.
     * This will ruin the decay information, but that is not supposed to be
     * a problem for the classification.
     */
    std::shuffle(begin(uev.incoming), end(uev.incoming), ran);
    std::shuffle(begin(uev.outgoing), end(uev.outgoing), ran);
    const HEJ::Event ev{std::move(uev), jet_def, min_jet_pt};

    if(ev.type() != expected){
      using HEJ::event_type::names;
      writer.hepeup = reader.hepeup;
      std::cerr << "wrong classification of event:\n";
      writer.writeEvent();
      std::cerr << "classified as " << names[ev.type()]
                << ", is " << names[expected] << '\n';
      return EXIT_FAILURE;
    }
  }

}
